#*************************************************************************************************************************************
#** Notification before use																											**
#*************************************************************************************************************************************
#** Depending on the type of product you are using, the definitions of Parameter, IO Logic, AxisStatus, etc. may be different.		**
#** This example is based on Ezi-SERVO, so please apply the appropriate value depending on the product you are using.				**
#*************************************************************************************************************************************
#** ex)	FM_EZISERVO_PARAM			// Parameter enum when using Ezi-SERVO						 									**
#**		FM_EZIMOTIONLINK_PARAM		// Parameter enum when using Ezi-MOTIONLINK														**
#*************************************************************************************************************************************

import sys
import os

try:
    library_path = os.path.abspath(
        os.path.join(os.path.dirname(__file__), "..", "..", "Library")
    )
except NameError:
    library_path = os.path.abspath(
        os.path.join(os.getcwd(), "..", "..", "Library")
    )
sys.path.append(library_path)

from FAS_EziMOTIONPlusR import *
from MOTION_DEFINE import *
from ReturnCodes_Define import *


def Connect(sPort: int, dwBaudRate: int, nPortID: int) -> bool:
    bSuccess = False

    if FAS_Connect(sPort, dwBaudRate, nPortID) == False:
        print("Connection Fail!")
        bSuccess = False

    else:
        print("Conncetion Success!")
        bSuccess = True
    return bSuccess


def CheckDriveInfo(sPort: int, nPortID: int, iSlaveNo: int) -> bool:
    status_result, byType, version = FAS_GetSlaveInfo(nPortID, iSlaveNo)

    if status_result != FMM_OK:
        print("Function(FAS_GetSlaveInfo) was failed.")
        return False
    print(
        f"Port {sPort} / Slave No {iSlaveNo} : TYPE= {byType}, Version= {version}"
    )
    return True


def main():
    sPort = "ttyS0"
    iSlaveNo = 0
    dwBaudRate = 115200
    nPortID = 0

    # Device Connect
    if not Connect(sPort, dwBaudRate, nPortID):
        input("Press Enter to exit...")
        exit(1)

    # Check Drive information
    if not CheckDriveInfo(sPort, nPortID, iSlaveNo):
        input("Press Enter to exit...")
        exit(1)

    # Connection Close
    FAS_Close(nPortID)

    input("Press Enter to exit...")


if __name__ == "__main__":
    main()
