#*************************************************************************************************************************************
#** Notification before use																											**
#*************************************************************************************************************************************
#** Depending on the type of product you are using, the definitions of Parameter, IO Logic, AxisStatus, etc. may be different.		**
#** This example is based on Ezi-SERVO, so please apply the appropriate value depending on the product you are using.				**
#*************************************************************************************************************************************
#** ex)	FM_EZISERVO_PARAM			// Parameter enum when using Ezi-SERVO						 									**
#**		FM_EZIMOTIONLINK_PARAM		// Parameter enum when using Ezi-MOTIONLINK														**
#*************************************************************************************************************************************

import sys
import os

try:
    library_path = os.path.abspath(
        os.path.join(os.path.dirname(__file__), "..", "..", "Library")
    )
except NameError:
    library_path = os.path.abspath(
        os.path.join(os.getcwd(), "..", "..", "Library")
    )
sys.path.append(library_path)

from FAS_EziMOTIONPlusR import *
from MOTION_DEFINE import *
from ReturnCodes_Define import *
from MOTION_EziSERVO_DEFINE import *


def Connect(sPort: int, dwBaudRate: int, nPortID: int) -> bool:
    bSuccess = False

    if FAS_Connect(sPort, dwBaudRate, nPortID) == False:
        print("Connection Fail!")
        bSuccess = False

    else:
        print("Conncetion Success!")
        bSuccess = True
    return bSuccess


def SetParameter(nPortID: int, iSlaveNo: int) -> bool:
    nChangeValue = 100

    print(
        "-----------------------------------------------------------------------------"
    )
    # Check The Axis Start Speed Parameter Status
    status_result, lParamVal = FAS_GetParameter(nPortID, iSlaveNo, SERVO_AXISSTARTSPEED)
    if status_result != FMM_OK:
        print("Function(FAS_GetParameter) was failed.")
        return False
    else:
        print(f"Load Parameter[Before] : Start Speed = {lParamVal}[pps]")

    print(
        "-----------------------------------------------------------------------------"
    )
    # Change the (Axxis Start Speed Parameter) vlaue to (nChangeValue) value.
    status_result = FAS_SetParameter(
        nPortID, iSlaveNo, SERVO_AXISSTARTSPEED, nChangeValue
    )
    if status_result != FMM_OK:
        print("Function(FAS_SetParameter) was failed.")
        return False
    else:
        print(f"Set Parameter: Start Speed = {nChangeValue}[pps]")

    print(
        "-----------------------------------------------------------------------------"
    )
    # Check the changed Axis Start Speed Parameter again.
    status_result, lParamVal = FAS_GetParameter(nPortID, iSlaveNo, SERVO_AXISSTARTSPEED)
    if status_result != FMM_OK:
        print("Function(FAS_GetParameter) was failed.")
        return False
    else:
        print(f"Load Parameter[After] : Start Speed = {lParamVal}[pps]")

    return True


def main():
    sPort = "ttyS0"
    iSlaveNo = 0
    dwBaudRate = 115200
    nPortID = 0

    # Device Connect
    if not Connect(sPort, dwBaudRate, nPortID):
        input("Press Enter to exit...")
        exit(1)

    # Load and Set Parameter
    if not SetParameter(nPortID, iSlaveNo):
        print("Failed to set parameter.")

    # Connection Close
    FAS_Close(nPortID)

    input("Press Enter to exit...")


if __name__ == "__main__":
    main()
