#*************************************************************************************************************************************
#** Notification before use																											**
#*************************************************************************************************************************************
#** Depending on the type of product you are using, the definitions of Parameter, IO Logic, AxisStatus, etc. may be different.		**
#** This example is based on Ezi-SERVO, so please apply the appropriate value depending on the product you are using.				**
#*************************************************************************************************************************************
#** ex)	FM_EZISERVO_PARAM			// Parameter enum when using Ezi-SERVO						 									**
#**		FM_EZIMOTIONLINK_PARAM		// Parameter enum when using Ezi-MOTIONLINK														**
#*************************************************************************************************************************************

import sys
import os

try:
    library_path = os.path.abspath(
        os.path.join(os.path.dirname(__file__), "..", "..", "Library")
    )
except NameError:
    library_path = os.path.abspath(
        os.path.join(os.getcwd(), "..", "..", "Library")
    )
sys.path.append(library_path)

from FAS_EziMOTIONPlusR import *
from MOTION_DEFINE import *
from ReturnCodes_Define import *
from MOTION_EziSERVO_DEFINE import *

import time

INPUTPIN = 12


def Connect(sPort: int, dwBaudRate: int, nPortID: int) -> bool:
    bSuccess = False

    if FAS_Connect(sPort, dwBaudRate, nPortID) == False:
        print("Connection Fail!")
        bSuccess = False

    else:
        print("Conncetion Success!")
        bSuccess = True
    return bSuccess


def SetInputPin(nPortID: int, iSlaveNo: int) -> bool:
    print(
        "-----------------------------------------------------------------------------"
    )

    # Set Input pin Value.
    byPinNo = 3
    byLevel = LEVEL_LOW_ACTIVE
    dwInputMask = SERVO_IN_BITMASK_USERIN0

    if FAS_SetIOAssignMap(nPortID, iSlaveNo, byPinNo, dwInputMask, byLevel) != FMM_OK:
        print("Function(FAS_SetIOAssignMap) was failed.")
        return False
    else:
        print(
            f"SERVO_IN_BITMASK_USERIN0 (Pin{byPinNo}) : [{'Low Active' if byLevel == LEVEL_LOW_ACTIVE else 'High Active'}]"
        )

    print(
        "-----------------------------------------------------------------------------"
    )

    # Show Input pins status
    for i in range(INPUTPIN):
        status_result, dwLogicMask, byLevel = FAS_GetIOAssignMap(nPortID, iSlaveNo, i)
        if status_result != FMM_OK:
            print("Function(FAS_GetIOAssignMap) was failed.")
            return False
        if dwLogicMask != IN_LOGIC_NONE:
            print(
                f"Input PIN[{i}] : Logic Mask 0x{dwLogicMask:08x} ({'Low Active' if byLevel == LEVEL_LOW_ACTIVE else 'High Active'})"
            )

        else:
            print(f"Input Pin[{i}] : Not Assigned")

    return True


def CheckInputSignal(nPortID: int, iSlaveNo: int) -> bool:
    dwInputMask = SERVO_IN_BITMASK_USERIN0

    print(
        "-----------------------------------------------------------------------------"
    )
    # When the value SERVO_IN_BITMASK_USERIN0 is entered with the set PinNo, an input confirmation message is displyed.
    # Monitoring input signal for 60 seconds
    for i in range(600):
        status_result, dwInput = FAS_GetIOInput(nPortNo, iSlaveNo)
        if status_result == FMM_OK:
            if dwInput & dwInputMask:
                print("INPUT PIN DETECTED.")
        else:
            print("Function(FAS_GetIOInput) was failed.")
            return False
        time.sleep(0.1)
    print("finish WaitSecond!")
    return True
    

def main(): 
    sPort = "ttyS0"
    iSlaveNo = 0
    dwBaudRate = 115200
    nPortID = 0

    # Device Connect
    if not Connect(sPort, dwBaudRate, nPortID):
        input("Press Enter to exit...")
        exit(1)

    # Set Input pins
    if not SetInputPin(nPortID, iSlaveNo):
        input("Press Enter to exit...")
        exit(1)

    # Check Input Pin Signal
    if not CheckInputSignal(nPortID, iSlaveNo):
        input("Press Enter to exit...")
        exit(1)

    # Connection Close
    FAS_Close(nPortID)
    input("Press Enter to exit...")


if __name__ == "__main__":
    main()
