#*************************************************************************************************************************************
#** Notification before use																											**
#*************************************************************************************************************************************
#** Depending on the type of product you are using, the definitions of Parameter, IO Logic, AxisStatus, etc. may be different.		**
#** This example is based on Ezi-SERVO, so please apply the appropriate value depending on the product you are using.				**
#*************************************************************************************************************************************
#** ex)	FM_EZISERVO_PARAM			// Parameter enum when using Ezi-SERVO						 									**
#**		FM_EZIMOTIONLINK_PARAM		// Parameter enum when using Ezi-MOTIONLINK														**
#*************************************************************************************************************************************

import sys
import os

try:
    library_path = os.path.abspath(
        os.path.join(os.path.dirname(__file__), "..", "..", "Library")
    )
except NameError:
    library_path = os.path.abspath(
        os.path.join(os.getcwd(), "..", "..", "Library")
    )
sys.path.append(library_path)

from FAS_EziMOTIONPlusR import *
from MOTION_DEFINE import *
from ReturnCodes_Define import *
from MOTION_EziSERVO_DEFINE import *


def Connect(sPort: int, dwBaudRate: int, nPortID: int) -> bool:
    bSuccess = False

    if FAS_Connect(sPort, dwBaudRate, nPortID) == False:
        print("Connection Fail!")
        bSuccess = False

    else:
        print("Conncetion Success!")
        bSuccess = True
    return bSuccess


def SetTrigger(nPortID: int, iSlaveNo: int) -> bool:
    Trg_Info = TRIGGER_INFO()
    uCount = 0

    # Set Trigger value
    cOutputNo = 0  # Output pin #0
    Trg_Info.wCount = 20  # Set the number of Trigger outputs
    Trg_Info.wOnTime = 250  # On Time Setting : 250 [ms]
    Trg_Info.wPeriod = 500  # Set Trigger period : 500 [ms]

    if FAS_SetTrigger(nPortID, iSlaveNo, cOutputNo, Trg_Info) != FMM_OK:
        print("Function(FAS_SetTrigger) was failed.")
        return False

    else:
        print("FAS_SetTrigger Success!")

    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	#~~ Fastech's product which has Input (or Output) only (Ezi-IO-I16, Ezi-IO-O16, Ezi-IO-I32, Ezi-IO-O32...), BitMask of Input (or Output) starts from 0x01,	~~
	#~~ However, for Input+Output mixed products (Ezi-IO-I8O8, Ezi-IO-I16O16, etc.), the BitMask of Output is allocated after the BitMasks of Input.			~~
	#~~ Ezi-IO-I8O8, for example, the BitMask of Input 0 is 0x0001, and the BitMask of Output 0 is 0x0100.														~~
	#~~ For detailed allocation methods, please refer to Section 2.3 of the "doc.md" file of this example.														~~
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	#~~ Fastech의 Input/Output 단일 제품 (Ezi-IO-I16, Ezi-IO-O16, Ezi-IO-I32, Ezi-IO-O32 등)은 Input과 Output의 BitMask가 각각 0x01부터 시작하지만,					~~
	#~~ Input+Output 혼합 제품 (Ezi-IO-I8O8, Ezi-IO-I16O16 등)은 Input의 BitMask를 전부 할당한 뒤, Output의 BitMask가 이어서 할당되어있습니다.						~~
	#~~ 예를들어 Ezi-IO-I8O8의 Input 0번의 BitMask는 0x0001, Output 0번의 BitMask는 0x0100입니다.																	~~
	#~~ 자세한 할당 방식은 본 예제의 "doc.md"파일 2.3절을 참고 부탁드립니다.																							~~
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    # Run Output
    uRunMask = 0x01 << cOutputNo  # Run Output Pin #0
    uStopMask = 0x00000000
    if FAS_SetRunStop(nPortID, iSlaveNo, uRunMask, uStopMask) != FMM_OK:
        print("Function(FAS_SetRunStop) was failed.")
        return False
    else:
        print("FAS_SetRunStop Success!")

    while True:
        time.sleep(0.1)

        status_result, uCount = FAS_GetTriggerCount(nPortID, iSlaveNo, cOutputNo)
        if status_result != FMM_OK:
            print("Function(FAS_GetTriggerCount) was failed.")
            return False
        else:
            print(f"Get Trigger [{uCount}] count")

        if Trg_Info.wCount <= uCount:
            break
    return True


def main():
    sPort = "ttyS0"
    iSlaveNo = 0
    dwBaudRate = 115200
    nPortID = 0

    # Device Connect
    if not Connect(sPort, dwBaudRate, nPortID):
        input("Press Enter to exit...")
        exit(1)

    # Trigger Set and Check Count
    if not SetTrigger(nPortID, iSlaveNo):
        input("Press Enter to exit...")
        exit(1)

    # Connection Close
    FAS_Close(nPortID)
    input("Press Enter to exit...")


if __name__ == "__main__":
    main()
