# -*- coding: utf-8 -*-
from ctypes import *
import ctypes
from FAS_EziMOTIONPlusR_Internal import *
from MOTION_DEFINE import *


# ------------------------------------------------------------------------------
# 					Connection Functions
# ------------------------------------------------------------------------------
# def FAS_Connect(sPort: int, dwBaud: int, nPortID: int) -> bool:
def FAS_Connect(sPort, dwBaud, nPortID):
    return FAS_Connect_Original(to_unicode(sPort), ctypes.c_ulong(dwBaud), ctypes.c_ubyte(nPortID))


# def FAS_OpenPort(sPort: int, dwBaud: int, nPortID : int) -> bool:
def FAS_OpenPort(sPort, dwBaud, nPortID):
    return FAS_OpenPort_Original(to_unicode(sPort), ctypes.c_ulong(dwBaud), ctypes.c_ubyte(nPortID))


# def FAS_AttachSlave(nPortID: int, iSlaveNo: int) -> bool:
def FAS_AttachSlave(nPortID, iSlaveNo):
    return FAS_AttachSlave_Original(ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo))


# def FAS_Close(nPortID: int):
def FAS_Close(nPortID):
    FAS_Close_Original(ctypes.c_ubyte(nPortID))


# def FAS_IsSlaveExist(nPortID: int, iSlaveNo: int) -> bool:
def FAS_IsSlaveExist(nPortID, iSlaveNo):
    return FAS_IsSlaveExist_Original(ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo))


# ------------------------------------------------------------------------------
# 					Log Functions
# ------------------------------------------------------------------------------
# def FAS_EnableLog(bEnable: bool):
def FAS_EnableLog(bEnable):
    FAS_EnableLog_Original(ctypes.c_int(bEnable))


# def FAS_SetLogLevel(level: int):
def FAS_SetLogLevel(level):
    FAS_SetLogLevel_Original(ctypes.c_int(level))


# def FAS_SetLogPath(lpPath: str) -> bool:
def FAS_SetLogPath(lpPath):
    return FAS_SetLogPath_Original(to_unicode(lpPath))


# def FAS_PrintCustomLog(nPortID: int, level: int, lpszMsg: str):
def FAS_PrintCustomLog(nPortID, level, lpszMsg):
    FAS_PrintCustomLog_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_int(level), to_unicode(lpszMsg)
    )


# ------------------------------------------------------------------------------
# 					Info Functions
# ------------------------------------------------------------------------------
# def FAS_GetSlaveInfo(nPortID: int, iSlaveNo: int) -> tuple[int, int, str]:
def FAS_GetSlaveInfo(nPortID, iSlaveNo):
    pType = ctypes.c_ubyte()
    lpBuff = ctypes.create_string_buffer(256)

    result = FAS_GetSlaveInfo_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.byref(pType),
        lpBuff,
        256
    )

    if result == 0:
        return result, pType.value, lpBuff.value.decode("utf-8")
    else:
        return result, None, None


# def FAS_GetMotorInfo(nPortID: int, iSlaveNo: int) -> tuple[int, int, str]:
def FAS_GetMotorInfo(nPortID, iSlaveNo):
    pType = ctypes.c_ubyte()
    lpBuff = ctypes.create_string_buffer(256)
    
    result = FAS_GetMotorInfo_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.byref(pType),
        lpBuff,
        256
    )
    if result == 0:
        return result, pType.value, lpBuff.value.decode("utf-8")
    else:
        return result, None, None


# def FAS_GetSlaveInfoEx(nPortID: int, iSlaveNo: int) -> tuple[int, str]:
def FAS_GetSlaveInfoEx(nPortID, iSlaveNo):
    drive_info = CDrive_Info()
    result = FAS_GetSlaveInfoEx_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), byref(drive_info)
    )
    if result == 0:
        lpDriveInfo = CDrive_Info_to_Drive_Info(drive_info)
        return result, lpDriveInfo
    else:
        return result, None


# ------------------------------------------------------------------------------
# 					Parameter Functions
# ------------------------------------------------------------------------------
# def FAS_SaveAllParameters(nPortID: int, iSlaveNo: int) -> int:
def FAS_SaveAllParameters(nPortID, iSlaveNo):
    return FAS_SaveAllParameters_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo)
    )

#def FAS_SetParameter(nPortID: int, iSlaveNo: int, iParamNo: int, lParamValue: int) -> int:
def FAS_SetParameter(nPortID, iSlaveNo, iParamNo, lParamValue):
    return FAS_SetParameter_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ubyte(iParamNo),
        ctypes.c_long(lParamValue)
    )


# def FAS_GetParameter(nPortID: int, iSlaveNo: int, iParamNo: int) -> tuple[int, int]:
def FAS_GetParameter(nPortID, iSlaveNo, iParamNo):
    lParamValue = ctypes.c_long()
    result = FAS_GetParameter_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ubyte(iParamNo),
        ctypes.byref(lParamValue)
    )
    if result == 0:
        return result, lParamValue.value
    else:
        return result, None


# def FAS_GetROMParameter(nPortID: int, iSlaveNo: int, iParamNo: int) -> tuple[int, int]:
def FAS_GetROMParameter(nPortID, iSlaveNo, iParamNo):
    lRomParam = ctypes.c_long()
    result = FAS_GetROMParameter_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ubyte(iParamNo),
        ctypes.byref(lRomParam)
    )
    if result == 0:
        return result, lRomParam.value
    else:
        return result, None


# ------------------------------------------------------------------------------
# 					IO Functions
# ------------------------------------------------------------------------------
# def FAS_SetIOInput(nPortID: int, iSlaveNo: int, dwIOSETMask: int, dwIOCLRMask: int) -> int:
def FAS_SetIOInput(nPortID, iSlaveNo, dwIOSETMask, dwIOCLRMask):
    return FAS_SetIOInput_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ulong(dwIOSETMask),
        ctypes.c_ulong(dwIOCLRMask)
    )


# def FAS_GetIOInput(nPortID: int, iSlaveNo: int) -> tuple[int, int]:
def FAS_GetIOInput(nPortID, iSlaveNo):
    dwIOInput = ctypes.c_ulong()
    result = FAS_GetIOInput_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.byref(dwIOInput)
    )
    if result == 0:
        return result, dwIOInput.value
    else:
        return result, None

# def FAS_SetIOOutput(nPortID: int, iSlaveNo: int, dwIOSETMask: int, dwIOCLRMask: int) -> int: 
def FAS_SetIOOutput(nPortID, iSlaveNo, dwIOSETMask, dwIOCLRMask):
    return FAS_SetIOOutput_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ulong(dwIOSETMask),
        ctypes.c_ulong(dwIOCLRMask)
    )


# def FAS_GetIOOutput(nPortID: int, iSlaveNo: int) -> tuple[int, int]:
def FAS_GetIOOutput(nPortID, iSlaveNo):
    dwIOOutput = ctypes.c_ulong()
    result = FAS_GetIOOutput_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.byref(dwIOOutput)
    )
    if result == 0:
        return result, dwIOOutput.value
    else:
        return result, None

# def FAS_GetIOAssignMap(nPortID: int, iSlaveNo: int, iIOPinNo: int) -> tuple[int, int, int]:
def FAS_GetIOAssignMap(nPortID, iSlaveNo, iIOPinNo):
    dwIOLogicMask = ctypes.c_ulong()
    bLevel = ctypes.c_ubyte()
    result = FAS_GetIOAssignMap_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ubyte(iIOPinNo),
        ctypes.byref(dwIOLogicMask),
        ctypes.byref(bLevel)
    )
    if result == 0:
        return result, dwIOLogicMask.value, bLevel.value
    else:
        return result, None, None

# def FAS_SetIOAssignMap(nPortID: int, iSlaveNo: int, iIOPinNo: int, dwIOLogicMask: int, bLevel: int) -> int:
def FAS_SetIOAssignMap(nPortID, iSlaveNo, iIOPinNo, dwIOLogicMask, bLevel):
    return FAS_SetIOAssignMap_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ubyte(iIOPinNo),
        ctypes.c_ulong(dwIOLogicMask),
        ctypes.c_ubyte(bLevel)
    )


# def FAS_IOAssignMapReadROM(nPortID: int, iSlaveNo: int) -> int:
def FAS_IOAssignMapReadROM(nPortID, iSlaveNo):
    return FAS_IOAssignMapReadROM_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo)
    )


# ------------------------------------------------------------------------------
# 					Servo Driver Control Functions
# ------------------------------------------------------------------------------
# def FAS_ServoEnable(nPortID: int, iSlaveNo: int, bOnOff: int) -> int:
def FAS_ServoEnable(nPortID, iSlaveNo, bOnOff):
    return FAS_ServoEnable_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.c_int(bOnOff)
    )


# def FAS_ServoAlarmReset(nPortID: int, iSlaveNo: int) -> int:
def FAS_ServoAlarmReset(nPortID, iSlaveNo):
    return FAS_ServoAlarmReset_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo)
    )


# def FAS_StepAlarmReset(nPortID: int, iSlaveNo: int, bReset: bool) -> int:
def FAS_StepAlarmReset(nPortID, iSlaveNo, bReset):
    return FAS_StepAlarmReset_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.c_int(bReset)
    )


# def FAS_BrakeSet(nPortID: int, iSlaveNo: int, bSet: int) -> tuple[int, int]:
def FAS_BrakeSet(nPortID, iSlaveNo, bSet):
    nResult = ctypes.c_int()
    result = FAS_BrakeSet_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_int(bSet),
        ctypes.byref(nResult)
    )
    if result == 0:
        return result, nResult.value
    else:
        return result, None


# ------------------------------------------------------------------------------
# 					Read Status and Position
# ------------------------------------------------------------------------------
# def FAS_GetAxisStatus(nPortID: int, iSlaveNo: int) -> tuple[int, int]:
def FAS_GetAxisStatus(nPortID, iSlaveNo):
    dwAxisStatus = ctypes.c_ulong()
    result = FAS_GetAxisStatus_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.byref(dwAxisStatus)
    )
    if result == 0:
        return result, dwAxisStatus.value
    else:
        return result, None


# def FAS_GetIOAxisStatus(nPortID: int, iSlaveNo: int) -> tuple[int, int, int, int]:
def FAS_GetIOAxisStatus(nPortID, iSlaveNo):
    dwInStatus = ctypes.c_ulong()
    dwOutStatus = ctypes.c_ulong()
    dwAxisStatus = ctypes.c_ulong()

    result = FAS_GetIOAxisStatus_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.byref(dwInStatus),
        ctypes.byref(dwOutStatus),
        ctypes.byref(dwAxisStatus)
    )
    if result == 0:
        return result, dwInStatus.value, dwOutStatus.value, dwAxisStatus.value
    else:
        return result, None, None, None

# def FAS_GetMotionStatus(nPortID: int, iSlaveNo: int) -> tuple[int, int, int, int, int, int]:
def FAS_GetMotionStatus(nPortID, iSlaveNo):
    lCmdPos = ctypes.c_long()
    lActPos = ctypes.c_long()
    lPosErr = ctypes.c_long()
    lActVel = ctypes.c_long()
    wPosItemNo = ctypes.c_ushort()

    result = FAS_GetMotionStatus_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.byref(lCmdPos),
        ctypes.byref(lActPos),
        ctypes.byref(lPosErr),
        ctypes.byref(lActVel),
        ctypes.byref(wPosItemNo)
    )
    if result == 0:
        return (
            result,
            lCmdPos.value,
            lActPos.value,
            lPosErr.value,
            lActVel.value,
            wPosItemNo.value
        )
    else:
        return result, None, None, None, None, None

# def FAS_GetAllStatus(nPortID: int, iSlaveNo: int) -> tuple[int, int, int, int, int, int, int, int, int]:
def FAS_GetAllStatus(nPortID, iSlaveNo):
    dwInStatus = ctypes.c_ulong()
    dwOutStatus = ctypes.c_ulong()
    dwAxisStatus = ctypes.c_ulong()
    lCmdPos = ctypes.c_long()
    lActPos = ctypes.c_long()
    lPosErr = ctypes.c_long()
    lActVel = ctypes.c_long()
    wPosItemNo = ctypes.c_ushort()

    result = FAS_GetAllStatus_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.byref(dwInStatus),
        ctypes.byref(dwOutStatus),
        ctypes.byref(dwAxisStatus),
        ctypes.byref(lCmdPos),
        ctypes.byref(lActPos),
        ctypes.byref(lPosErr),
        ctypes.byref(lActVel),
        ctypes.byref(wPosItemNo)
    )
    if result == 0:
        return (
            result,
            dwInStatus.value,
            dwOutStatus.value,
            dwAxisStatus.value,
            lCmdPos.value,
            lActPos.value,
            lPosErr.value,
            lActVel.value,
            wPosItemNo.value
        )
    else:
        return result, None, None, None, None, None, None, None, None

# def FAS_GetAllStatusEx(nPortID: int, iSlaveNo: int, pTypes: list) -> tuple[int, list]:
def FAS_GetAllStatusEx(nPortID, iSlaveNo, pTypes):
    pTypesarray = (ctypes.c_ubyte * len(pTypes))(*pTypes)
    pDatas = (ctypes.c_int * 12)()
    result = FAS_GetAllStatusEx_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        pTypesarray, ctypes.cast(pDatas, ctypes.POINTER(ctypes.c_int))
    )
    if result == 0:
        return result, list(pDatas)
    else:
        return result, None


# def FAS_SetCommandPos(nPortID: int, iSlaveNo: int, lCmdPos: int) -> int:
def FAS_SetCommandPos(nPortID, iSlaveNo, lCmdPos):
    return FAS_SetCommandPos_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.c_long(lCmdPos)
    )


# def FAS_SetActualPos(nPortID: int, iSlaveNo: int, lActPos: int) -> int:
def FAS_SetActualPos(nPortID, iSlaveNo, lActPos):
    return FAS_SetActualPos_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.c_long(lActPos)
    )


# def FAS_ClearPosition(nPortID: int, iSlaveNo: int) -> int:
def FAS_ClearPosition(nPortID, iSlaveNo):
    return FAS_ClearPosition_Original(ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo))


# def FAS_GetCommandPos(nPortID: int, iSlaveNo: int) -> tuple[int, int]:
def FAS_GetCommandPos(nPortID, iSlaveNo):
    lCmdPos = ctypes.c_long()
    result = FAS_GetCommandPos_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.byref(lCmdPos)
    )
    if result == 0:
        return result, lCmdPos.value
    else:
        return result, None


# def FAS_GetActualPos(nPortID: int, iSlaveNo: int) -> tuple[int, int]:
def FAS_GetActualPos(nPortID, iSlaveNo):
    lActPos = ctypes.c_long()
    result = FAS_GetActualPos_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.byref(lActPos)
    )
    if result == 0:
        return result, lActPos.value
    else:
        return result, None


# def FAS_GetPosError(nPortID: int, iSlaveNo: int) -> tuple[int, int]:
def FAS_GetPosError(nPortID, iSlaveNo):
    lPosErr = ctypes.c_long()
    result = FAS_GetPosError_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.byref(lPosErr)
    )
    if result == 0:
        return result, lPosErr.value
    else:
        return result, None


# def FAS_GetActualVel(nPortID: int, iSlaveNo: int) -> tuple[int, int]:
def FAS_GetActualVel(nPortID, iSlaveNo):
    lActVel = ctypes.c_long()
    result = FAS_GetActualVel_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.byref(lActVel)
    )
    if result == 0:
        return result, lActVel.value
    else:
        return result, None


# def FAS_GetAlarmType(nPortID: int, iSlaveNo: int) -> tuple[int, int]:
def FAS_GetAlarmType(nPortID, iSlaveNo):
    nAlarmType = ctypes.c_ubyte()
    result = FAS_GetAlarmType_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.byref(nAlarmType)
    )
    if result == 0:
        return result, nAlarmType.value
    else:
        return result, None


# ------------------------------------------------------------------
# 					Motion Functions.
# ------------------------------------------------------------------
# def FAS_MoveStop(nPortID: int, iSlaveNo: int) -> int:
def FAS_MoveStop(nPortID, iSlaveNo):
    return FAS_MoveStop_Original(ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo))


# def FAS_EmergencyStop(nPortID: int, iSlaveNo: int) -> int:
def FAS_EmergencyStop(nPortID, iSlaveNo):
    return FAS_EmergencyStop_Original(ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo))


# def FAS_MovePause(nPortID: int, iSlaveNo: int, bPause: bool) -> int:
def FAS_MovePause(nPortID, iSlaveNo, bPause):
    return FAS_MovePause_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.c_int(bPause)
    )


# def FAS_MoveOriginSingleAxis(nPortID: int, iSlaveNo: int) -> int:
def FAS_MoveOriginSingleAxis(nPortID, iSlaveNo):
    return FAS_MoveOriginSingleAxis_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo)
    )

# def FAS_MoveSingleAxisAbsPos(nPortID: int, iSlaveNo: int, lAbsPos: int, lVelocity: int) -> int:
def FAS_MoveSingleAxisAbsPos(nPortID, iSlaveNo, lAbsPos, lVelocity):
    return FAS_MoveSingleAxisAbsPos_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_long(lAbsPos),
        ctypes.c_ulong(lVelocity)
    )

# def FAS_MoveSingleAxisIncPos(nPortID: int, iSlaveNo: int, lIncPos: int, lVelocity: int) -> int:
def FAS_MoveSingleAxisIncPos(nPortID, iSlaveNo, lIncPos, lVelocity):
    return FAS_MoveSingleAxisIncPos_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_long(lIncPos),
        ctypes.c_ulong(lVelocity)
    )


# def FAS_MoveToLimit(nPortID: int, iSlaveNo: int, lVelocity: int, iLimitDir: int) -> int:
def FAS_MoveToLimit(nPortID, iSlaveNo, lVelocity, iLimitDir):
    return FAS_MoveToLimit_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ulong(lVelocity),
        ctypes.c_int(iLimitDir)
    )


# def FAS_MoveVelocity(nPortID: int, iSlaveNo: int, lVelocity: int, iVelDir: int) -> int:
def FAS_MoveVelocity(nPortID, iSlaveNo, lVelocity, iVelDir):
    return FAS_MoveVelocity_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ulong(lVelocity),
        ctypes.c_int(iVelDir)
    )


# def FAS_PositionAbsOverride(nPortID: int, iSlaveNo: int, lOverridePos: int) -> int:
def FAS_PositionAbsOverride(nPortID, iSlaveNo, lOverridePos):
    return FAS_PositionAbsOverride_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.c_long(lOverridePos)
    )


# def FAS_PositionIncOverride(nPortID: int, iSlaveNo: int, lOverridePos: int) -> int:
def FAS_PositionIncOverride(nPortID, iSlaveNo, lOverridePos):
    return FAS_PositionIncOverride_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.c_long(lOverridePos)
    )


# def FAS_VelocityOverride(nPortID: int, iSlaveNo: int, lVelocity: int) -> int:
def FAS_VelocityOverride(nPortID, iSlaveNo, lVelocity):
    return FAS_VelocityOverride_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.c_ulong(lVelocity)
    )

# def FAS_MoveLinearAbsPos(nPortID: int, nNoOfSlaves: int, iSlavesNo: list, lplAbsPos: list, lFeedrate: int, wAccelTime: int) -> int:
def FAS_MoveLinearAbsPos(nPortID, nNoOfSlaves, iSlavesNo, lplAbsPos, lFeedrate, wAccelTime):
    iSlavesNoArray = (ctypes.c_ubyte * nNoOfSlaves)(*iSlavesNo)
    lplAbsPosArray = (ctypes.c_long * nNoOfSlaves)(*lplAbsPos)
    result = FAS_MoveLinearAbsPos_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(nNoOfSlaves),
        ctypes.cast(iSlavesNoArray, ctypes.POINTER(ctypes.c_ubyte)),
        ctypes.cast(lplAbsPosArray, ctypes.POINTER(ctypes.c_long)),
        ctypes.c_ulong(lFeedrate),
        ctypes.c_ushort(wAccelTime)
    )
    return result

# def FAS_MoveLinearIncPos(nPortID: int, nNoOfSlaves: int, iSlavesNo: list, lplIncPos: list, lFeedrate: int, wAccelTime: int) -> int:
def FAS_MoveLinearIncPos(nPortID, nNoOfSlaves, iSlavesNo, lplIncPos, lFeedrate, wAccelTime):
    iSlavesNoArray = (ctypes.c_ubyte * nNoOfSlaves)(*iSlavesNo)
    lplIncPosArray = (ctypes.c_long * nNoOfSlaves)(*lplIncPos)
    result = FAS_MoveLinearIncPos_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(nNoOfSlaves),
        ctypes.cast(iSlavesNoArray, ctypes.POINTER(ctypes.c_ubyte)),
        ctypes.cast(lplIncPosArray, ctypes.POINTER(ctypes.c_long)),
        ctypes.c_ulong(lFeedrate),
        ctypes.c_ushort(wAccelTime)
    )
    return result

# def FAS_MoveLinearAbsPos2(nPortID: int, nNoOfSlaves: int, iSlavesNo: list, lplAbsPos: list, lFeedrate: int, wAccelTime: int) -> int:
def FAS_MoveLinearAbsPos2(nPortID, nNoOfSlaves, iSlavesNo, lplAbsPos, lFeedrate, wAccelTime):
    iSlavesNoArray = (ctypes.c_ubyte * nNoOfSlaves)(*iSlavesNo)
    lplAbsPosArray = (ctypes.c_long * nNoOfSlaves)(*lplAbsPos)
    result = FAS_MoveLinearAbsPos2_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(nNoOfSlaves),
        ctypes.cast(iSlavesNoArray, ctypes.POINTER(ctypes.c_ubyte)),
        ctypes.cast(lplAbsPosArray, ctypes.POINTER(ctypes.c_long)),
        ctypes.c_ulong(lFeedrate),
        ctypes.c_ushort(wAccelTime)
    )
    return result

# def FAS_MoveLinearIncPos2(nPortID: int, nNoOfSlaves: int, iSlavesNo: list, lplIncPos: list, lFeedrate: int, wAccelTime: int) -> int:
def FAS_MoveLinearIncPos2(nPortID, nNoOfSlaves, iSlavesNo, lplIncPos, lFeedrate, wAccelTime):
    iSlavesNoArray = (ctypes.c_ubyte * nNoOfSlaves)(*iSlavesNo)
    lplIncPosArray = (ctypes.c_long * nNoOfSlaves)(*lplIncPos)
    result = FAS_MoveLinearIncPos2_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(nNoOfSlaves),
        ctypes.cast(iSlavesNoArray, ctypes.POINTER(ctypes.c_ubyte)),
        ctypes.cast(lplIncPosArray, ctypes.POINTER(ctypes.c_long)),
        ctypes.c_ulong(lFeedrate),
        ctypes.c_ushort(wAccelTime)
    )

    return result

# def FAS_MoveCircleAbsPos1(nPortID: int, nNoOfSlaves: int, iSlavesNo: list, lplCirEndAbs: list, lplCirCenterAbs: list, iDirection: int, lFeedrate: int, wAccelTime: int, bSCurve: int) -> int:
def FAS_MoveCircleAbsPos1(
    nPortID,
    nNoOfSlaves,
    iSlavesNo,
    lplCirEndAbs,
    lplCirCenterAbs,
    iDirection,
    lFeedrate,
    wAccelTime,
    bSCurve
):
    iSlavesNoArray = (ctypes.c_ubyte * nNoOfSlaves)(*iSlavesNo)
    lplCirEndAbsArray = (ctypes.c_long * nNoOfSlaves)(*lplCirEndAbs)
    lplCirCenterAbsArray = (ctypes.c_long * nNoOfSlaves)(*lplCirCenterAbs)
    result = FAS_MoveCircleAbsPos1_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(nNoOfSlaves),
        ctypes.cast(iSlavesNoArray, ctypes.POINTER(ctypes.c_ubyte)),
        ctypes.cast(lplCirEndAbsArray, ctypes.POINTER(ctypes.c_long)),
        ctypes.cast(lplCirCenterAbsArray,ctypes.POINTER(ctypes.c_long)),
        ctypes.c_int(iDirection),
        ctypes.c_ulong(lFeedrate),
        ctypes.c_ushort(wAccelTime),
        ctypes.c_int(bSCurve)
    )
    return result

# def FAS_MoveCircleIncPos1(nPortID: int, nNoOfSlaves: int, iSlavesNo: list, lplCirEndInc: list, lplCirCenterInc: list, iDirection: int, lFeedrate: int, wAccelTime: int, bSCurve: int) -> int:
def FAS_MoveCircleIncPos1(
    nPortID,
    nNoOfSlaves,
    iSlavesNo,
    lplCirEndInc,
    lplCirCenterInc,
    iDirection,
    lFeedrate,
    wAccelTime,
    bSCurve
):
    iSlavesNoArray = (ctypes.c_ubyte * nNoOfSlaves)(*iSlavesNo)
    lplCirEndIncArray = (ctypes.c_long * nNoOfSlaves)(*lplCirEndInc)
    lplCirCenterIncArray = (ctypes.c_long * nNoOfSlaves)(*lplCirCenterInc)
    result = FAS_MoveCircleIncPos1_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(nNoOfSlaves),
        ctypes.cast(iSlavesNoArray, ctypes.POINTER(ctypes.c_ubyte)),
        ctypes.cast(lplCirEndIncArray, ctypes.POINTER(ctypes.c_long)),
        ctypes.cast(lplCirCenterIncArray,ctypes.POINTER(ctypes.c_long)),
        ctypes.c_int(iDirection),
        ctypes.c_ulong(lFeedrate),
        ctypes.c_ushort(wAccelTime),
        ctypes.c_int(bSCurve)
    )
    return result

# def FAS_MoveCircleAbsPos2(nPortID: int, nNoOfSlaves: int, iSlavesNo: list, lplCirEndAbs: list, lRadius: int, iDirection: int, lFeedrate: int, wAccelTime: int, bSCurve: int) -> int:
def FAS_MoveCircleAbsPos2(
    nPortID,
    nNoOfSlaves,
    iSlavesNo,
    lplCirEndAbs,
    lRadius,
    iDirection,
    lFeedrate,
    wAccelTime,
    bSCurve
):
    iSlavesNoArray = (ctypes.c_ubyte * nNoOfSlaves)(*iSlavesNo)
    lplCirEndAbsArray = (ctypes.c_long * nNoOfSlaves)(*lplCirEndAbs)
    result = FAS_MoveCircleAbsPos2_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(nNoOfSlaves),
        ctypes.cast(iSlavesNoArray, ctypes.POINTER(ctypes.c_ubyte)),
        ctypes.cast(lplCirEndAbsArray, ctypes.POINTER(ctypes.c_long)),
        ctypes.c_ulong(lRadius),
        ctypes.c_int(iDirection),
        ctypes.c_ulong(lFeedrate),
        ctypes.c_ushort(wAccelTime),
        ctypes.c_int(bSCurve)
    )
    return result

# def FAS_MoveCircleIncPos2(nPortID: int, nNoOfSlaves: int, iSlavesNo: list, lplCirEndInc: list, lRadius: int, iDirection: int, lFeedrate: int, wAccelTime: int, bSCurve: int) -> int:
def FAS_MoveCircleIncPos2(
    nPortID,
    nNoOfSlaves,
    iSlavesNo,
    lplCirEndInc,
    lRadius,
    iDirection,
    lFeedrate,
    wAccelTime,
    bSCurve
):
    iSlavesNoArray = (ctypes.c_ubyte * nNoOfSlaves)(*iSlavesNo)
    lplCirEndIncArray = (ctypes.c_long * nNoOfSlaves)(*lplCirEndInc)
    result = FAS_MoveCircleIncPos2_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(nNoOfSlaves),
        ctypes.cast(iSlavesNoArray, ctypes.POINTER(ctypes.c_ubyte)),
        ctypes.cast(lplCirEndIncArray, ctypes.POINTER(ctypes.c_long)),
        ctypes.c_ulong(lRadius),
        ctypes.c_int(iDirection),
        ctypes.c_ulong(lFeedrate),
        ctypes.c_ushort(wAccelTime),
        ctypes.c_int(bSCurve)
    )
    return result

# def FAS_MoveCircleAbsPos3(nPortID: int, nNoOfSlaves: int, iSlavesNo: list, lplCirCenterAbs: list, nAngle: int, iDirection: int, lFeedrate: int, wAccelTime: int, bSCurve: int) -> int:
def FAS_MoveCircleAbsPos3(
    nPortID,
    nNoOfSlaves,
    iSlavesNo,
    lplCirCenterAbs,
    nAngle,
    iDirection,
    lFeedrate,
    wAccelTime,
    bSCurve
):
    iSlavesNoArray = (ctypes.c_ubyte * nNoOfSlaves)(*iSlavesNo)
    lplCirCenterAbsArray = (ctypes.c_long * nNoOfSlaves)(*lplCirCenterAbs)
    result = FAS_MoveCircleAbsPos3_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(nNoOfSlaves),
        ctypes.cast(iSlavesNoArray, ctypes.POINTER(ctypes.c_ubyte)),
        ctypes.cast(lplCirCenterAbsArray, ctypes.POINTER(ctypes.c_long)),
        ctypes.c_ulong(nAngle),
        ctypes.c_int(iDirection),
        ctypes.c_ulong(lFeedrate),
        ctypes.c_ushort(wAccelTime),
        ctypes.c_int(bSCurve)
    )
    return result

# def FAS_MoveCircleIncPos3(nPortID: int, nNoOfSlaves: int, iSlavesNo: list, lplCirCenterInc: list, nAngle: int, iDirection: int, lFeedrate: int, wAccelTime: int, bSCurve: int) -> int:
def FAS_MoveCircleIncPos3(
    nPortID,
    nNoOfSlaves,
    iSlavesNo,
    lplCirCenterInc,
    nAngle,
    iDirection,
    lFeedrate,
    wAccelTime,
    bSCurve
):
    iSlavesNoArray = (ctypes.c_ubyte * nNoOfSlaves)(*iSlavesNo)
    lplCirCenterAbsArray = (ctypes.c_long * nNoOfSlaves)(*lplCirCenterInc)
    result = FAS_MoveCircleIncPos3_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(nNoOfSlaves),
        ctypes.cast(iSlavesNoArray, ctypes.POINTER(ctypes.c_ubyte)),
        ctypes.cast(lplCirCenterAbsArray, ctypes.POINTER(ctypes.c_long)),
        ctypes.c_ulong(nAngle),
        ctypes.c_int(iDirection),
        ctypes.c_ulong(lFeedrate),
        ctypes.c_ushort(wAccelTime),
        ctypes.c_int(bSCurve)
    )
    return result

# def FAS_TriggerOutput_RunA(nPortID: int, iSlaveNo: int, bStartTrigger: int, lStartPos: int, dwPeriod: int, dwPulseTime: int) -> int:
def FAS_TriggerOutput_RunA(
    nPortID,
    iSlaveNo,
    bStartTrigger,
    lStartPos,
    dwPeriod,
    dwPulseTime
):
    return FAS_TriggerOutput_RunA_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_int(bStartTrigger),
        ctypes.c_long(lStartPos),
        ctypes.c_ulong(dwPeriod),
        ctypes.c_ulong(dwPulseTime)
    )


# def FAS_TriggerOutput_Status(nPortID: int, iSlaveNo: int) -> tuple[int, int]:
def FAS_TriggerOutput_Status(nPortID, iSlaveNo):
    bTriggerStatus = ctypes.c_ubyte()
    result = FAS_TriggerOutput_Status_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.byref(bTriggerStatus)
    )
    if result == 0:
        return result, bTriggerStatus.value
    else:
        return result, None

# def FAS_SetTriggerOutputEx(nPortID: int, iSlaveNo: int, nOutputNo: int, bRun: int, wOnTime: int, nTriggerCount: int, arrTriggerPosition: list) -> int:
def FAS_SetTriggerOutputEx(
    nPortID,
    iSlaveNo,
    nOutputNo,
    bRun,
    wOnTime,
    nTriggerCount,
    arrTriggerPosition,
):
    arrType = (ctypes.c_long * nTriggerCount)(*arrTriggerPosition)
    result = FAS_SetTriggerOutputEx_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ubyte(nOutputNo),
        ctypes.c_ubyte(bRun),
        ctypes.c_ushort(wOnTime),
        ctypes.c_ubyte(nTriggerCount),
        ctypes.cast(arrType, ctypes.POINTER(ctypes.c_long))
    )

    return result

# def FAS_GetTriggerOutputEx(nPortID: int, iSlaveNo: int, nOutputNo: int) -> tuple[int, int, int, int, List]:
def FAS_GetTriggerOutputEx(nPortID, iSlaveNo, nOutputNo):
    bRun = ctypes.c_ubyte()
    wOnTime = ctypes.c_ushort()
    nTriggerCount = ctypes.c_ubyte()
    arrTriggerPosition = ctypes.c_long()
    result = FAS_GetTriggerOutputEx_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ubyte(nOutputNo),
        ctypes.byref(bRun),
        ctypes.byref(wOnTime),
        ctypes.byref(nTriggerCount),
        ctypes.byref(arrTriggerPosition)
    )
    if result == 0:
        return (
            result,
            bRun.value,
            wOnTime.value,
            nTriggerCount.value,
            list(arrTriggerPosition)
        )
    else:
        return result, None, None, None, None

# def FAS_MovePush(nPortID: int,iSlaveNo: int,dwStartSpd: int,dwMoveSpd: int,lPosition: int,wAccel: int,wDecel: int,wPushRate: int,dwPushSpd: int,lEndPosition: int,wPushMode: int) -> int:
def FAS_MovePush(
    nPortID,
    iSlaveNo,
    dwStartSpd,
    dwMoveSpd,
    lPosition,
    wAccel,
    wDecel,
    wPushRate,
    dwPushSpd,
    lEndPosition,
    wPushMode
):
    return FAS_MovePush_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ulong(dwStartSpd),
        ctypes.c_ulong(dwMoveSpd),
        ctypes.c_long(lPosition),
        ctypes.c_ushort(wAccel),
        ctypes.c_ushort(wDecel),
        ctypes.c_ushort(wPushRate),
        ctypes.c_ulong(dwPushSpd),
        ctypes.c_long(lEndPosition),
        ctypes.c_ushort(wPushMode)
    )


# def FAS_GetPushStatus(nPortID: int, iSlaveNo: int) -> tuple[int, int]:
def FAS_GetPushStatus(nPortID, iSlaveNo):
    nPushStatus = ctypes.c_ubyte()
    result = FAS_GetPushStatus_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.byref(nPushStatus)
    )
    if result == 0:
        return result, nPushStatus.value
    else:
        return result, None

# ------------------------------------------------------------------
# 					Ex-Motion Functions.
# ------------------------------------------------------------------
# def FAS_MoveSingleAxisAbsPosEx(nPortID: int, iSlaveNo: int, lAbsPos: int, lVelocity: int, lpExOption: MOTION_OPTION_EX) -> int:
def FAS_MoveSingleAxisAbsPosEx(
    nPortID,
    iSlaveNo,
    lAbsPos,
    lVelocity,
    lpExOption
):

    opt = MOTION_OPTION_EX_to_CMOTION_OPTION_EX(lpExOption)
    result = FAS_MoveSingleAxisAbsPosEx_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_long(lAbsPos),
        ctypes.c_ulong(lVelocity),
        ctypes.byref(opt)
    )
    return result

# def FAS_MoveSingleAxisIncPosEx(nPortID: int, iSlaveNo: int, lIncPos: int, lVelocity: int, lpExOption: MOTION_OPTION_EX) -> int:
def FAS_MoveSingleAxisIncPosEx(
    nPortID,
    iSlaveNo,
    lIncPos,
    lVelocity,
    lpExOption
):
    opt = MOTION_OPTION_EX_to_CMOTION_OPTION_EX(lpExOption)
    result = FAS_MoveSingleAxisIncPosEx_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_long(lIncPos),
        ctypes.c_ulong(lVelocity),
        ctypes.byref(opt),
    )
    return result

# def FAS_MoveVelocityEx(nPortID: int, iSlaveNo: int, lVelocity: int, iVelDir: int, lpExOption: VELOCITY_OPTION_EX) -> int:
def FAS_MoveVelocityEx(
    nPortID,
    iSlaveNo,
    lVelocity,
    iVelDir,
    lpExOption
):
    opt = VELOCITY_OPTION_EX_to_CVELOCITY_OPTION_EX(lpExOption)
    result = FAS_MoveVelocityEx_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ulong(lVelocity),
        ctypes.c_int(iVelDir),
        ctypes.byref(opt)
    )
    return result


# ------------------------------------------------------------------
# 					All-Motion Functions.
# ------------------------------------------------------------------
# def FAS_AllMoveStop(nPortID: int) -> int:
def FAS_AllMoveStop(nPortID):
    return FAS_AllMoveStop_Original(ctypes.c_ubyte(nPortID))


# def FAS_AllEmergencyStop(nPortID: int) -> int:
def FAS_AllEmergencyStop(nPortID):
    return FAS_AllEmergencyStop_Original(ctypes.c_ubyte(nPortID))


# def FAS_AllMoveOriginSingleAxis(nPortID: int) -> int:
def FAS_AllMoveOriginSingleAxis(nPortID):
    return FAS_AllMoveOriginSingleAxis_Original(ctypes.c_ubyte(nPortID))


# def FAS_AllMoveSingleAxisAbsPos(nPortID: int, lAbsPos: int, lVelocity: int) -> int:
def FAS_AllMoveSingleAxisAbsPos(nPortID, lAbsPos, lVelocity):
    return FAS_AllMoveSingleAxisAbsPos_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_long(lAbsPos), ctypes.c_ulong(lVelocity)
    )


# def FAS_AllMoveSingleAxisIncPos(nPortID: int, lIncPos: int, lVelocity: int) -> int:
def FAS_AllMoveSingleAxisIncPos(nPortID, lIncPos, lVelocity):
    return FAS_AllMoveSingleAxisIncPos_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_long(lIncPos), ctypes.c_ulong(lVelocity)
    )


# ------------------------------------------------------------------
# 					Position Table Functions.
# ------------------------------------------------------------------
# def FAS_PosTableReadItem(nPortID: int, iSlaveNo: int, wItemNo: int) -> tuple[int, List]:
def FAS_PosTableReadItem(nPortID, iSlaveNo, wItemNo):
    Item = CITEM_NODE()
    result = FAS_PosTableReadItem_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ushort(wItemNo),
        byref(Item)
    )
    if result == 0:
        lpItem = CITEM_NODE_to_ITEM_NODE(Item)
        return result, lpItem
    else:
        return result, None

# def FAS_PosTableWriteItem(nPortID: int, iSlaveNo: int, wItemNo: int, lpItem: ITEM_NODE) -> int:
def FAS_PosTableWriteItem(nPortID, iSlaveNo, wItemNo, lpItem):
    Item = ITEM_NODE_to_CITEM_NODE(lpItem)
    result = FAS_PosTableWriteItem_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ushort(wItemNo),
        ctypes.byref(Item)
    )
    return result


# def FAS_PosTableWriteROM(nPortID: int, iSlaveNo: int) -> int:
def FAS_PosTableWriteROM(nPortID, iSlaveNo):
    return FAS_PosTableWriteROM_Original(ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo))


# def FAS_PosTableReadROM(nPortID: int, iSlaveNo: int) -> int:
def FAS_PosTableReadROM(nPortID, iSlaveNo):
    return FAS_PosTableReadROM_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo)
    )


# def FAS_PosTableRunItem(nPortID: int, iSlaveNo: int, wItemNo: int) -> int:
def FAS_PosTableRunItem(nPortID, iSlaveNo, wItemNo):
    return FAS_PosTableRunItem_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.c_ushort(wItemNo)
    )

# def FAS_PosTableReadOneItem(nPortID: int, iSlaveNo: int, wItemNo: int, wOffset: int) -> tuple[int, int]:
def FAS_PosTableReadOneItem(nPortID, iSlaveNo, wItemNo, wOffset):
    lPosItemVal = ctypes.c_long()
    result = FAS_PosTableReadOneItem_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ushort(wItemNo),
        ctypes.c_ushort(wOffset),
        ctypes.byref(lPosItemVal)
    )
    if result == 0:
        return result, lPosItemVal.value
    else:
        return result, None



# def FAS_PosTableWriteOneItem(nPortID: int, iSlaveNo: int, wItemNo: int, wOffset: int, lPosItemVal: int) -> int:
def FAS_PosTableWriteOneItem(nPortID, iSlaveNo, wItemNo, wOffset, lPosItemVal):
    return FAS_PosTableWriteOneItem_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ushort(wItemNo),
        ctypes.c_ushort(wOffset),
        ctypes.c_long(lPosItemVal)
    )

# def FAS_PosTableSingleRunItem(nPortID: int, iSlaveNo: int, bNextMove: int, wItemNo: int) -> int:
def FAS_PosTableSingleRunItem(nPortID, iSlaveNo, bNextMove, wItemNo):
    return FAS_PosTableSingleRunItem_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_int(bNextMove),
        ctypes.c_ushort(wItemNo)
    )


# ------------------------------------------------------------------
# 					Gap Control Functions.
# ------------------------------------------------------------------
# def FAS_GapControlEnable(nPortID: int, iSlaveNo: int, wItemNo: int, lGapCompSpeed: int, lGapAccTime: int, lGapDecTime: int, lGapStartSpeed: int) -> int:
def FAS_GapControlEnable(
    nPortID,
    iSlaveNo,
    wItemNo,
    lGapCompSpeed,
    lGapAccTime,
    lGapDecTime,
    lGapStartSpeed
):
    return FAS_GapControlEnable_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ushort(wItemNo),
        ctypes.c_long(lGapCompSpeed),
        ctypes.c_long(lGapAccTime),
        ctypes.c_long(lGapDecTime),
        ctypes.c_long(lGapStartSpeed)
    )


# def FAS_GapControlDisable(nPortID: int, iSlaveNo: int) -> int:
def FAS_GapControlDisable(nPortID, iSlaveNo):
    return FAS_GapControlDisable_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo)
    )


# def FAS_IsGapControlEnable(nPortID: int, iSlaveNo: int) -> tuple[int, int, int]:
def FAS_IsGapControlEnable(nPortID, iSlaveNo):
    bIsEnable = ctypes.c_int()
    wCurrentItemNo = ctypes.c_ushort()
    result = FAS_IsGapControlEnable_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.byref(bIsEnable),
        ctypes.byref(wCurrentItemNo)
    )
    if result == 0:
        return result, bIsEnable.value, wCurrentItemNo.value
    else:
        return result, None, None


# def FAS_GapControlGetADCValue(nPortID: int, iSlaveNo: int) -> tuple[int, int]:
def FAS_GapControlGetADCValue(nPortID, iSlaveNo):
    lADCValue = ctypes.c_long()
    result = FAS_GapControlGetADCValue_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.byref(lADCValue)
    )
    if result == 0:
        return result, lADCValue.value
    else:
        return result, None

# def FAS_GapOneResultMonitor(nPortID: int, iSlaveNo: int) -> tuple[int, int, int, int, int, int, int, int]:
def FAS_GapOneResultMonitor(nPortID, iSlaveNo):
    bUpdated = ctypes.c_ubyte()
    iIndex = ctypes.c_long()
    lGapValue = ctypes.c_long()
    lCmdPos = ctypes.c_long()
    lActPos = ctypes.c_long()
    lCompValue = ctypes.c_long()
    lReserved = ctypes.c_long()
    result = FAS_GapOneResultMonitor_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.byref(bUpdated),
        ctypes.byref(iIndex),
        ctypes.byref(lGapValue),
        ctypes.byref(lCmdPos),
        ctypes.byref(lActPos),
        ctypes.byref(lCompValue),
        ctypes.byref(lReserved)
    )
    if result == 0:
        return (
            result,
            bUpdated.value,
            iIndex.value,
            lGapValue.value,
            lCmdPos.value,
            lActPos.value,
            lCompValue.value,
            lReserved.value
        )
    else:
        return result, None, None, None, None, None, None, None


# ------------------------------------------------------------------
# 					Alarm Type History Functions.
# ------------------------------------------------------------------
# def FAS_GetAlarmLogs(nPortID: int, iSlaveNo: int) -> tuple[int, List]:
def FAS_GetAlarmLogs(nPortID, iSlaveNo):
    AlarmLog = CALARM_LOG()
    result = FAS_GetAlarmLogs_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.byref(AlarmLog)
    )
    if result == 0:
        pAlarmLog = CALARM_LOG_to_ALARM_LOG(AlarmLog)
        return result, pAlarmLog
    else:
        return result, None


# def FAS_ResetAlarmLogs(nPortID: int, iS5laveNo: int) -> int:
def FAS_ResetAlarmLogs(nPortID, iSlaveNo):
    return FAS_ResetAlarmLogs_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo)
    )


# ------------------------------------------------------------------
# 					I/O Module Functions.
# ------------------------------------------------------------------
# def FAS_GetInput(nPortID: int, iSlaveNo: int) -> tuple[int, int, int]:
def FAS_GetInput(nPortID, iSlaveNo):
    uInput = ctypes.c_ulong()
    uLatch = ctypes.c_ulong()
    result = FAS_GetInput_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.byref(uInput),
        ctypes.byref(uLatch)
    )
    if result == 0:
        return result, uInput.value, uLatch.value
    else:
        return result, None, None


# def FAS_ClearLatch(nPortID: int, iSlaveNo: int, uLatchMask: int) -> int:
def FAS_ClearLatch(nPortID, iSlaveNo, uLatchMask):
    return FAS_ClearLatch_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.c_ulong(uLatchMask)
    )


# def FAS_GetLatchCount(nPortID: int, iSlaveNo: int, iInputNo: int) -> tuple[int, int]:
def FAS_GetLatchCount(nPortID, iSlaveNo, iInputNo):
    uCount = ctypes.c_ulong()
    result = FAS_GetLatchCount_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ubyte(iInputNo),
        ctypes.byref(uCount)
    )
    if result == 0:
        return result, uCount.value
    else:
        return result, None


# def FAS_GetLatchCountAll(nPortID: int, iSlaveNo: int) -> tuple[int, list]:
def FAS_GetLatchCountAll(nPortID, iSlaveNo):
    ppuAllCount = (ctypes.c_ulong * 16)()
    result = FAS_GetLatchCountAll_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.byref(ppuAllCount)
    )
    if result == 0:
        return result, list(ppuAllCount)
    else:
        return result, None


# def FAS_GetLatchCountAll32(nPortID: int, iSlaveNo: int) -> tuple[int, list]:
def FAS_GetLatchCountAll32(nPortID, iSlaveNo):
    ppuAllCount = (ctypes.c_ulong * 32)()
    result = FAS_GetLatchCountAll32_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.byref(ppuAllCount)
    )
    if result == 0:
        return result, list(ppuAllCount)
    else:
        return result, None


# def FAS_ClearLatchCount(nPortID: int, iSlaveNo: int, uInputMask: int) -> int:
def FAS_ClearLatchCount(nPortID, iSlaveNo, uInputMask):
    return FAS_ClearLatchCount_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.c_ulong(uInputMask)
    )


# def FAS_GetOutput(nPortID: int, iSlaveNo: int) -> tuple[int, int, int]:
def FAS_GetOutput(nPortID, iSlaveNo):
    uOutput = ctypes.c_ulong()
    uStatus = ctypes.c_ulong()
    result = FAS_GetOutput_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.byref(uOutput),
        ctypes.byref(uStatus)
    )
    if result == 0:
        return result, uOutput.value, uStatus.value
    else:
        return result, None, None


# def FAS_SetOutput(nPortID: int, iSlaveNo: int, uSetMask: int, uClearMask: int) -> int:
def FAS_SetOutput(nPortID, iSlaveNo, uSetMask, uClearMask):
    return FAS_SetOutput_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ulong(uSetMask),
        ctypes.c_ulong(uClearMask)
    )

# def FAS_SetTrigger(nPortID: int, iSlaveNo: int, uOutputNo: int, pTrigger: TRIGGER_INFO) -> int:
def FAS_SetTrigger(nPortID, iSlaveNo, uOutputNo, pTrigger):
    Trigger = TRIGGER_INFO_to_CTRIGGER_INFO(pTrigger)
    result = FAS_SetTrigger_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ubyte(uOutputNo),
        ctypes.byref(Trigger)
    )
    return result


# def FAS_SetRunStop(nPortID: int, iSlaveNo: int, uRunMask: int, uStopMask: int) -> int:
def FAS_SetRunStop(nPortID, iSlaveNo, uRunMask, uStopMask):
    return FAS_SetRunStop_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ulong(uRunMask),
        ctypes.c_ulong(uStopMask)
    )


# def FAS_GetTriggerCount(nPortID: int, iSlaveNo: int, uOutputNo: int) -> tuple[int, int]:
def FAS_GetTriggerCount(nPortID, iSlaveNo, uOutputNo):
    uCount = ctypes.c_ulong()
    result = FAS_GetTriggerCount_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ubyte(uOutputNo),
        ctypes.byref(uCount)
    )
    if result == 0:
        return result, uCount.value
    else:
        return result, None


# def FAS_GetIOLevel(nPortID: int, iSlaveNo: int) -> tuple[int, int]:
def FAS_GetIOLevel(nPortID, iSlaveNo):
    uIOLevel = ctypes.c_ulong()
    result = FAS_GetIOLevel_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.byref(uIOLevel)
    )
    if result == 0:
        return result, uIOLevel.value
    else:
        return result, None


# def FAS_SetIOLevel(nPortID: int, iSlaveNo: int, uIOLevel: int) -> int:
def FAS_SetIOLevel(nPortID, iSlaveNo, uIOLevel):
    return FAS_SetIOLevel_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.c_ulong(uIOLevel)
    )


# def FAS_LoadIOLevel(nPortID: int, iSlaveNo: int) -> int:
def FAS_LoadIOLevel(nPortID, iSlaveNo):
    return FAS_LoadIOLevel_Original(ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo))


# def FAS_SaveIOLevel(nPortID: int, iSlaveNo: int) -> int:
def FAS_SaveIOLevel(nPortID, iSlaveNo):
    return FAS_SaveIOLevel_Original(ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo))


# def FAS_GetInputFilter(nPortID: int, iSlaveNo: int) -> tuple[int, int]:
def FAS_GetInputFilter(nPortID, iSlaveNo):
    filter = ctypes.c_ushort()
    result = FAS_GetInputFilter_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.byref(filter)
    )
    if result == 0:
        return result, filter.value
    else:
        return result, None


# def FAS_SetInputFilter(nPortID: int, iSlaveNo: int, filter: int) -> int:
def FAS_SetInputFilter(nPortID, iSlaveNo, filter):
    return FAS_SetInputFilter_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ushort(filter)
    )


# def FAS_GetIODirection(nPortID: int, iSlaveNo: int) -> tuple[int, int]:
def FAS_GetIODirection(nPortID, iSlaveNo):
    direction = ctypes.c_ulong()
    result = FAS_GetIODirection_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.byref(direction)
    )
    if result == 0:
        return result, direction.value
    else:
        return result, None


# def FAS_SetIODirection(nPortID: int, iSlaveNo: int, direction: int) -> int:
def FAS_SetIODirection(nPortID, iSlaveNo, direction):
    return FAS_SetIODirection_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.c_ulong(direction)
    )


# ------------------------------------------------------------------
# 					Ezi-IO AD Functions
# ------------------------------------------------------------------
# def FAS_SetADConfig(nPortID: int, iSlaveNo: int, channel: int, type: int, value: int) -> tuple[int,int]:
def FAS_SetADConfig(nPortID, iSlaveNo, channel, type, value):
    recv = ctypes.c_long()
    result = FAS_SetADConfig_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ubyte(channel),
        ctypes.c_int(type),
        ctypes.c_long(value),
        ctypes.byref(recv))
    if result == 0:
        return result, recv.value
    else:
        return result, None

    

# def FAS_GetADConfig(nPortID: int, iSlaveNo: int, channel: int, type: int) -> tuple[int, int]:
def FAS_GetADConfig(nPortID, iSlaveNo, channel, type):
    value = ctypes.c_long()
    result = FAS_GetADConfig_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ubyte(channel),
        ctypes.c_int(type),
        ctypes.byref(value)
    )
    if result == 0:
        return result, value.value
    else:
        return result, None


# def FAS_LoadADConfig(nPortID: int, iSlaveNo: int) -> int:
def FAS_LoadADConfig(nPortID, iSlaveNo):
    return FAS_LoadADConfig_Original(ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo))


# def FAS_SaveADConfig(nPortID: int, iSlaveNo: int) -> int:
def FAS_SaveADConfig(nPortID, iSlaveNo):
    return FAS_SaveADConfig_Original(ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo))


# def FAS_ReadADValue(nPortID: int, iSlaveNo: int, channel: int) -> tuple[int, int]:
def FAS_ReadADValue(nPortID, iSlaveNo, channel):
    advalue = ctypes.c_short()
    result = FAS_ReadADValue_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ubyte(channel),
        ctypes.byref(advalue)
    )
    if result == 0:
        return result, advalue.value
    else:
        return result, None


# def FAS_ReadADAllValue(nPortID: int, iSlaveNo: int, offset: int) -> tuple[int, list]:
def FAS_ReadADAllValue(nPortID, iSlaveNo, offset):
    adbuffer = (ctypes.c_short * 8)()
    result = FAS_ReadADAllValue_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ubyte(offset),
        adbuffer
    )
    if result == 0:
        return result, list(adbuffer)
    else:
        return result, None


# def FAS_GetAllADResult(nPortID: int, iSlaveNo: int) -> tuple[int, int]:
def FAS_GetAllADResult(nPortID, iSlaveNo):
    result_array = AD_RESULT()
    result = FAS_GetAllADResult_Original(
        ctypes.c_ubyte(nPortID), ctypes.c_ubyte(iSlaveNo), ctypes.byref(result_array)
    )
    if result == 0:
        return result, list(result_array.elements)
    else:
        return result, None

# def FAS_GetADResult(nPortID: int, iSlaveNo: int, channel: int) -> tuple[int, float]:  
def FAS_GetADResult(nPortID, iSlaveNo, channel):  
    adresult = ctypes.c_float()
    result = FAS_GetADResult_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ubyte(channel),
        ctypes.byref(adresult)
    )
    if result == 0:
        return result, adresult.value
    else:
        return result, None


# def FAS_SetADRange(nPortID: int, iSlaveNo: int, channel: int, range: int) -> int:
def FAS_SetADRange(nPortID, iSlaveNo, channel, range):
    return FAS_SetADRange_Original(
        ctypes.c_ubyte(nPortID),
        ctypes.c_ubyte(iSlaveNo),
        ctypes.c_ubyte(channel),
        ctypes.c_int(range),
    )

